#include <EEPROM.h>
#include <Wire.h>
#include <hd44780.h>                       // main hd44780 header
#include <hd44780ioClass/hd44780_I2Cexp.h> // i2c expander i/o class header
hd44780_I2Cexp lcd; // declare lcd object: auto locate & auto config expander chip

// Buttons
const int btnPage = 4;
const int btnTab = 5;
const int btnUp = 6;
const int btnDown = 7;

// Control Pins
const int pinHeat = 9;
const int pinLEDS = 10;
const int pinPump = 11;

// Sensor Pins
const int thermistorPin = A0;
const int thermistorVPin = 2;
const int thermistorGPin = 3;

// LCD geometry
const int LCD_COLS = 20;
const int LCD_ROWS = 4;
    
// Intervals and States for Timing
bool waterON = LOW; //water state
bool lightON = LOW; // light state
bool heatON = LOW; // heat state

unsigned long waterLastON; // when was water last triggered millis()
unsigned long lightLastON; // when was heat last triggered millis()

byte waterFlows; // how many floods
byte waterDuration;  // how long floods minutes
int waterPWM; // how much power to the pump

byte lightDuration; // low long light hours
int lightPWM; // how   much power to light

byte tempDegC; // what temp
byte tempRange; // vary by how much -ve before reheating
byte currentTemp = 0; // What is the last reported temperature?

byte currentPage = 0; // what page is currently being displayed? 0=summary 1=stats 2=water 3=light 4=heat
byte currentTab = 0; // tab index used for scrolling through items on page
bool URQ = true; // set to true to update the values in the menus
bool ERQ = false; // set to true to update the values in the EEPROM


// Menu stuff is abstracted into menus.h, so let's include that.
#include "menus.h"

void setup()
{
  // setup the input and output pins
  pinMode(btnPage, INPUT_PULLUP);
  pinMode(btnTab, INPUT_PULLUP);
  pinMode(btnUp, INPUT_PULLUP);
  pinMode(btnDown, INPUT_PULLUP);
  pinMode(pinHeat, OUTPUT); 
  pinMode(pinLEDS, OUTPUT); 
  pinMode(pinPump, OUTPUT); 
  pinMode(thermistorVPin, OUTPUT);
  pinMode(thermistorGPin, OUTPUT);
  
  //voltage pins
  digitalWrite(thermistorVPin, HIGH);
  digitalWrite(thermistorGPin, LOW);
  
  // read the EEPROM values into RAM
  waterFlows = EEPROM.read(ESTORE_waterFlows);
  waterDuration = EEPROM.read(ESTORE_waterDuration);
  waterPWM = EEPROM.read(ESTORE_waterPWM); 
  lightDuration = EEPROM.read(ESTORE_lightDuration); 
  lightPWM = EEPROM.read(ESTORE_lightPWM); 
  tempDegC = EEPROM.read(ESTORE_tempDegC); 
  tempRange = EEPROM.read(ESTORE_tempRange);

  // fire up the LCD
  lcd.begin(LCD_COLS, LCD_ROWS);
  show(boot);
  for (int i = 0; i<=100;i++){
    analogWrite(pinLEDS, map(i,0,100,0,255));
    delay(10);
  }
  delay(1000);
  for (int i = 100; i>=0;i--){
    analogWrite(pinLEDS, map(i,0,100,0,255));
    delay(10);
  }
  show(summary);

  // set the timers 
  waterLastON = millis();
  lightLastON = millis();
}

void waterTimerCheck(){
  unsigned long now = millis();
  
  // should we turn the water on? Convert hours to milliseconds and check
  if (now-waterLastON >= (24/waterFlows)*3600000){
    waterON = HIGH;
    waterLastON = now;
  }

  // should we turn the water off? Convert minutes to milliseconds and check
  if (now-waterLastON >= (waterDuration*60000)){
    waterON = LOW;
  }
}

void lightTimerCheck(){
  unsigned long now = millis();
  
  // should we turn the water on? Convert hours to milliseconds and check
  if (now-lightLastON >= (24-lightDuration)*3600000){
    lightON = HIGH;
    lightLastON = now;
  }

  // should we turn the water off? Convert minutes to milliseconds and check
  if (now-lightLastON >= (lightDuration*3600000)){
    lightON = LOW;
  }
}
 
void heatCheck(){
  // tempDegC
  // tempRange
  double tempC;

  // turn on the power to the thermistor
  digitalWrite(thermistorVPin, HIGH);
  digitalWrite(thermistorGPin, LOW);
  delay(50);
  
  // read the thermistor
  int rawADC = analogRead(A0);

  // turn off the power to the thermisor
  digitalWrite(thermistorVPin, LOW);
  digitalWrite(thermistorGPin, LOW);

  
  tempC = log(((10240000/rawADC) - 10000));
  tempC = 1 / (0.001129148 + (0.000234125 + (0.0000000876741 *tempC *tempC ))*tempC );
  tempC -= 273.15;
  currentTemp = (byte)tempC;

  if (tempC <= tempDegC-tempRange){
    heatON = HIGH;
  } else if (tempC >= tempDegC){
    heatON = LOW;
  }
  }

void setPinStates(){
  // set the pin states based on waterON, lightON, heatON
  digitalWrite(pinHeat, heatON);
  if (lightON){
        analogWrite(pinLEDS, map(lightPWM,0,100,0,255));
  }
    if (waterON){
        analogWrite(pinPump, map(waterPWM,0,100,0,255));
  }
  }

  
void loop() {
  // deal with display and buttons in menu.h
  doLoopTasks();

  // check the timers
  lightTimerCheck();
  waterTimerCheck();
  heatCheck();
  
  // set the pins
  setPinStates();
  
  delay(300);
}

